@@ -68,6 +68,9 @@ struct dm_task {
 	int deferred_remove;
 	int enable_checks;
 	int expected_errno;
+	int ioctl_errno;
+
+	int record_timestamp;
 
 	char *uuid;
 	char *mangled_uuid;
