@@ -14,12 +14,32 @@
 
 #include "dmlib.h"
 
+#include "math.h" /* log10() */
+
 #define DM_STATS_REGION_NOT_PRESENT UINT64_MAX
 
+#define NSEC_PER_USEC   1000L
 #define NSEC_PER_MSEC   1000000L
 #define NSEC_PER_SEC    1000000000L
 
 #define PRECISE_ARG "precise_timestamps"
+#define HISTOGRAM_ARG "histogram:"
+
+/* Histogram bin */
+struct dm_histogram_bin {
+	uint64_t upper; /* Upper bound on this bin. */
+	uint64_t count; /* Count value for this bin. */
+};
+
+struct dm_histogram {
+	/* The stats handle this histogram belongs to. */
+	const struct dm_stats *dms;
+	/* The region this histogram belongs to. */
+	const struct dm_stats_region *region;
+	uint64_t sum; /* Sum of histogram bin counts. */
+	int nr_bins; /* Number of histogram bins assigned. */
+	struct dm_histogram_bin bins[0];
+};
 
 /*
  * See Documentation/device-mapper/statistics.txt for full descriptions
@@ -39,6 +59,7 @@ struct dm_stats_counters {
 	uint64_t weighted_io_nsecs; /* Weighted num milliseconds doing I/Os */
 	uint64_t total_read_nsecs;  /* Total time spent reading in milliseconds */
 	uint64_t total_write_nsecs; /* Total time spent writing in milliseconds */
+	struct dm_histogram *histogram; /* Histogram. */
 };
 
 struct dm_stats_region {
@@ -49,6 +70,7 @@ struct dm_stats_region {
 	char *program_id;
 	char *aux_data;
 	uint64_t timescale; /* precise_timestamps is per-region */
+	struct dm_histogram *bounds; /* histogram configuration */
 	struct dm_stats_counters *counters;
 };
 
@@ -60,6 +82,7 @@ struct dm_stats {
 	char *uuid; /* device-mapper UUID */
 	char *program_id; /* default program_id for this handle */
 	struct dm_pool *mem; /* memory pool for region and counter tables */
+	struct dm_pool *hist_mem; /* separate pool for histogram tables */
 	uint64_t nr_regions; /* total number of present regions */
 	uint64_t max_region; /* size of the regions table */
 	uint64_t interval_ns;  /* sampling interval in nanoseconds */
@@ -82,7 +105,7 @@ static char *_program_id_from_proc(void)
 
 	if (!fgets(buf, sizeof(buf), comm)) {
 		log_error("Could not read from %s", PROC_SELF_COMM);
-		if(fclose(comm))
+		if (fclose(comm))
 			stack;
 		return NULL;
 	}
@@ -93,20 +116,52 @@ static char *_program_id_from_proc(void)
 	return dm_strdup(buf);
 }
 
+static uint64_t _nr_areas(uint64_t len, uint64_t step)
+{
+	/* Default is one area. */
+	if (!len || !step)
+		return 1;
+	/*
+	 * drivers/md/dm-stats.c::message_stats_create()
+	 * A region may be sub-divided into areas with their own counters.
+	 * Any partial area at the end of the region is treated as an
+	 * additional complete area.
+	 */
+	return (len + step - 1) / step;
+}
+
+static uint64_t _nr_areas_region(struct dm_stats_region *region)
+{
+	return _nr_areas(region->len, region->step);
+}
+
 struct dm_stats *dm_stats_create(const char *program_id)
 {
+	size_t hist_hint = sizeof(struct dm_histogram_bin);
 	struct dm_stats *dms = NULL;
 
 	if (!(dms = dm_zalloc(sizeof(*dms))))
 		return_NULL;
-	if (!(dms->mem = dm_pool_create("stats_pool", 4096)))
-		goto_out;
+
+	/* FIXME: better hint. */
+	if (!(dms->mem = dm_pool_create("stats_pool", 4096))) {
+		dm_free(dms);
+		return_NULL;
+	}
+
+	if (!(dms->hist_mem = dm_pool_create("histogram_pool", hist_hint)))
+		goto_bad;
 
 	if (!program_id || !strlen(program_id))
 		dms->program_id = _program_id_from_proc();
 	else
 		dms->program_id = dm_strdup(program_id);
 
+	if (!dms->program_id) {
+		dm_pool_destroy(dms->hist_mem);
+		goto_bad;
+	}
+
 	dms->major = -1;
 	dms->minor = -1;
 	dms->name = NULL;
@@ -121,7 +176,9 @@ struct dm_stats *dm_stats_create(const char *program_id)
 	dms->regions = NULL;
 
 	return dms;
-out:
+
+bad:
+	dm_pool_destroy(dms->mem);
 	dm_free(dms);
 	return NULL;
 }
@@ -134,6 +191,20 @@ static int _stats_region_present(const struct dm_stats_region *region)
 	return !(region->region_id == DM_STATS_REGION_NOT_PRESENT);
 }
 
+static void _stats_histograms_destroy(struct dm_pool *mem,
+				      struct dm_stats_region *region)
+{
+	/* Unpopulated handle. */
+	if (!region->counters)
+		return;
+
+	/*
+	 * Free everything in the pool back to the first histogram.
+	 */
+	if (region->counters[0].histogram)
+		dm_pool_free(mem, region->counters[0].histogram);
+}
+
 static void _stats_region_destroy(struct dm_stats_region *region)
 {
 	if (!_stats_region_present(region))
@@ -142,6 +213,8 @@ static void _stats_region_destroy(struct dm_stats_region *region)
 	/**
 	 * Don't free counters here explicitly; it will be dropped
 	 * from the pool along with the corresponding regions table.
+	 *
+	 * The following objects are all allocated with dm_malloc.
 	 */
 
 	if (region->program_id)
@@ -159,8 +232,11 @@ static void _stats_regions_destroy(struct dm_stats *dms)
 		return;
 
 	/* walk backwards to obey pool order */
-	for (i = dms->max_region; (i != DM_STATS_REGION_NOT_PRESENT); i--)
+	for (i = dms->max_region; (i != DM_STATS_REGION_NOT_PRESENT); i--) {
+		_stats_histograms_destroy(dms->hist_mem, &dms->regions[i]);
 		_stats_region_destroy(&dms->regions[i]);
+	}
+
 	dm_pool_free(mem, dms->regions);
 }
 
@@ -234,6 +310,7 @@ static int _stats_check_precise_timestamps(const struct dm_stats *dms)
 	/* Already checked? */
 	if (dms && dms->precise)
 		return 1;
+
 	return dm_message_supports_precise_timestamps();
 }
 
@@ -242,6 +319,109 @@ int dm_stats_driver_supports_precise(void)
 	return _stats_check_precise_timestamps(NULL);
 }
 
+int dm_stats_driver_supports_histogram(void)
+{
+	return _stats_check_precise_timestamps(NULL);
+}
+
+static int _fill_hist_arg(char *hist_arg, size_t hist_len, uint64_t scale,
+			  struct dm_histogram *bounds)
+{
+	int i, l, len = 0, nr_bins;
+	char *arg = hist_arg;
+	uint64_t value;
+
+	nr_bins = bounds->nr_bins;
+
+	for (i = 0; i < nr_bins; i++) {
+		value = bounds->bins[i].upper / scale;
+		if ((l = dm_snprintf(arg, hist_len - len, FMTu64"%s", value,
+				     (i == (nr_bins - 1)) ? "" : ",")) < 0)
+			return_0;
+		len += l;
+		arg += l;
+	}
+	return 1;
+}
+
+static void *_get_hist_arg(struct dm_histogram *bounds, uint64_t scale,
+			   size_t *len)
+{
+	struct dm_histogram_bin *entry, *bins;
+	size_t hist_len = 1; /* terminating '\0' */
+	double value;
+
+	entry = bins = bounds->bins;
+
+	entry += bounds->nr_bins - 1;
+	while(entry >= bins) {
+		value = (double) (entry--)->upper;
+		/* Use lround to avoid size_t -> double cast warning. */
+		hist_len += 1 + (size_t) lround(log10(value / scale));
+		if (entry != bins)
+			hist_len++; /* ',' */
+	}
+
+	*len = hist_len;
+
+	return dm_zalloc(hist_len);
+}
+
+static char *_build_histogram_arg(struct dm_histogram *bounds, int *precise)
+{
+	struct dm_histogram_bin *entry, *bins;
+	size_t hist_len;
+	char *hist_arg;
+	uint64_t scale;
+
+	entry = bins = bounds->bins;
+
+	/* Empty histogram is invalid. */
+	if (!bounds->nr_bins) {
+		log_error("Cannot format empty histogram description.");
+		return NULL;
+	}
+
+	/* Validate entries and set *precise if precision < 1ms. */
+	entry += bounds->nr_bins - 1;
+	while (entry >= bins) {
+		if (entry != bins) {
+			if (entry->upper < (entry - 1)->upper) {
+				log_error("Histogram boundaries must be in "
+					  "order of increasing magnitude.");
+				return 0;
+			}
+		}
+
+		/*
+		 * Only enable precise_timestamps automatically if any
+		 * value in the histogram bounds uses precision < 1ms.
+		 */
+		if (((entry--)->upper % NSEC_PER_MSEC) && !*precise)
+			*precise = 1;
+	}
+
+	scale = (*precise) ? 1 : NSEC_PER_MSEC;
+
+	/* Calculate hist_len and allocate a character buffer. */
+	if (!(hist_arg = _get_hist_arg(bounds, scale, &hist_len))) {
+		log_error("Could not allocate memory for histogram argument.");
+		return 0;
+	}
+
+	/* Fill hist_arg with boundary strings. */
+	if (!_fill_hist_arg(hist_arg, hist_len, scale, bounds))
+		goto_bad;
+
+	return hist_arg;
+
+bad:
+	log_error("Could not build histogram arguments.");
+	dm_free(hist_arg);
+
+	return NULL;
+}
+
 static struct dm_task *_stats_send_message(struct dm_stats *dms, char *msg)
 {
 	struct dm_task *dmt;
@@ -250,21 +430,132 @@ static struct dm_task *_stats_send_message(struct dm_stats *dms, char *msg)
 		return_0;
 
 	if (!_set_stats_device(dms, dmt))
-		goto_out;
+		goto_bad;
 
 	if (!dm_task_set_message(dmt, msg))
-		goto_out;
+		goto_bad;
 
 	if (!dm_task_run(dmt))
-		goto_out;
+		goto_bad;
 
 	return dmt;
-out:
+
+bad:
 	dm_task_destroy(dmt);
 	return NULL;
 }
 
-static int _stats_parse_list_region(struct dm_stats_region *region, char *line)
+/*
+ * Parse a histogram specification returned by the kernel in a
+ * @stats_list response.
+ */
+static int _stats_parse_histogram_spec(struct dm_stats *dms,
+				       struct dm_stats_region *region,
+				       const char *histogram)
+{
+	static const char *_valid_chars = "0123456789,";
+	uint64_t scale = region->timescale, this_val = 0;
+	struct dm_pool *mem = dms->hist_mem;
+	struct dm_histogram_bin cur;
+	struct dm_histogram hist;
+	int nr_bins = 1;
+	const char *c, *v, *val_start;
+	char *p, *endptr = NULL;
+
+	/* Advance past "histogram:". */
+	histogram = strchr(histogram, ':');
+	if (!histogram) {
+		log_error("Could not parse histogram description.");
+		return 0;
+	}
+	histogram++;
+
+	/* @stats_list rows are newline terminated. */
+	if ((p = strchr(histogram, '\n')))
+		*p = '\0';
+
+	if (!dm_pool_begin_object(mem, sizeof(cur)))
+		return_0;
+
+	memset(&hist, 0, sizeof(hist));
+
+	hist.nr_bins = 0; /* fix later */
+	hist.region = region;
+	hist.dms = dms;
+
+	if (!dm_pool_grow_object(mem, &hist, sizeof(hist)))
+		goto_bad;
+
+	c = histogram;
+	do {
+		for (v = _valid_chars; *v; v++)
+			if (*c == *v)
+				break;
+		if (!*v) {
+			stack;
+			goto badchar;
+		}
+
+		if (*c == ',') {
+			log_error("Invalid histogram description: %s",
+				  histogram);
+			goto bad;
+		} else {
+			val_start = c;
+			endptr = NULL;
+
+			this_val = strtoull(val_start, &endptr, 10);
+			if (!endptr) {
+				log_error("Could not parse histogram boundary.");
+				goto bad;
+			}
+
+			c = endptr; /* Advance to units, comma, or end. */
+
+			if (*c == ',')
+				c++;
+			else if (*c || (*c == ' ')) { /* Expected ',' or NULL. */
+				stack;
+				goto badchar;
+			}
+
+			if (*c == ',')
+				c++;
+
+			cur.upper = scale * this_val;
+			cur.count = 0;
+
+			if (!dm_pool_grow_object(mem, &cur, sizeof(cur)))
+				goto_bad;
+
+			nr_bins++;
+		}
+	} while (*c && (*c != ' '));
+
+	/* final upper bound. */
+	cur.upper = UINT64_MAX;
+	if (!dm_pool_grow_object(mem, &cur, sizeof(cur)))
+		goto_bad;
+
+	region->bounds = dm_pool_end_object(mem);
+
+	if (!region->bounds)
+		return_0;
+
+	region->bounds->nr_bins = nr_bins;
+
+	log_debug("Added region histogram spec with %d entries.", nr_bins);
+	return 1;
+
+badchar:
+	log_error("Invalid character in histogram: '%c' (0x%x)", *c, *c);
+bad:
+	dm_pool_abandon_object(mem);
+	return 0;
+}
+
+static int _stats_parse_list_region(struct dm_stats *dms,
+				    struct dm_stats_region *region, char *line)
 {
 	char *p = NULL, string_data[4096]; /* FIXME: add dm_sscanf with %ms? */
 	const char *program_id, *aux_data, *stats_args;
@@ -316,6 +607,12 @@ static int _stats_parse_list_region(struct dm_stats_region *region, char *line)
 	else
 		region->timescale = NSEC_PER_MSEC;
 
+	if ((p = strstr(stats_args, HISTOGRAM_ARG))) {
+		if (!_stats_parse_histogram_spec(dms, region, p))
+			return_0;
+	} else
+		region->bounds = NULL;
+
 	if (!(region->program_id = dm_strdup(program_id)))
 		return_0;
 	if (!(region->aux_data = dm_strdup(aux_data))) {
@@ -329,11 +626,11 @@ static int _stats_parse_list_region(struct dm_stats_region *region, char *line)
 
 static int _stats_parse_list(struct dm_stats *dms, const char *resp)
 {
-	struct dm_pool *mem = dms->mem;
-	struct dm_stats_region cur;
 	uint64_t max_region = 0, nr_regions = 0;
+	struct dm_stats_region cur, fill;
+	struct dm_pool *mem = dms->mem;
 	FILE *list_rows;
-	/* FIXME: determine correct maximum line length based on kernel format */
+	/* FIXME: use correct maximum line length for kernel format */
 	char line[256];
 
 	if (!resp) {
@@ -359,26 +656,25 @@ static int _stats_parse_list(struct dm_stats *dms, const char *resp)
 		return_0;
 
 	if (!dm_pool_begin_object(mem, 1024))
-		goto_out;
+		goto_bad;
 
 	while(fgets(line, sizeof(line), list_rows)) {
 
-		if (!_stats_parse_list_region(&cur, line))
-			goto_out;
+		if (!_stats_parse_list_region(dms, &cur, line))
+			goto_bad;
 
 		/* handle holes in the list of region_ids */
 		if (cur.region_id > max_region) {
-			struct dm_stats_region fill;
 			memset(&fill, 0, sizeof(fill));
 			fill.region_id = DM_STATS_REGION_NOT_PRESENT;
 			do {
 				if (!dm_pool_grow_object(mem, &fill, sizeof(fill)))
-					goto_out;
+					goto_bad;
 			} while (max_region++ < (cur.region_id - 1));
 		}
 
 		if (!dm_pool_grow_object(mem, &cur, sizeof(cur)))
-			goto_out;
+			goto_bad;
 
 		max_region++;
 		nr_regions++;
@@ -392,10 +688,12 @@ static int _stats_parse_list(struct dm_stats *dms, const char *resp)
 		stack;
 
 	return 1;
-out:
-	if(fclose(list_rows))
+
+bad:
+	if (fclose(list_rows))
 		stack;
 	dm_pool_abandon_object(mem);
+
 	return 0;
 }
 
@@ -420,25 +718,108 @@ int dm_stats_list(struct dm_stats *dms, const char *program_id)
 	}
 
 	if (!(dmt = _stats_send_message(dms, msg)))
-		return 0;
+		return_0;
 
 	if (!_stats_parse_list(dms, dm_task_get_message_response(dmt))) {
 		log_error("Could not parse @stats_list response.");
-		goto out;
+		goto bad;
 	}
 
 	dm_task_destroy(dmt);
 	return 1;
 
-out:
+bad:
 	dm_task_destroy(dmt);
 	return 0;
 }
 
-static int _stats_parse_region(struct dm_pool *mem, const char *resp,
+/*
+ * Parse histogram data returned from a @stats_print operation.
+ */
+static int _stats_parse_histogram(struct dm_pool *mem, char *hist_str,
+				  struct dm_histogram **histogram,
+				  struct dm_stats_region *region)
+{
+	struct dm_histogram hist, *bounds = region->bounds;
+	static const char *_valid_chars = "0123456789:";
+	int nr_bins = region->bounds->nr_bins;
+	const char *c, *v, *val_start;
+	struct dm_histogram_bin cur;
+	uint64_t sum = 0, this_val;
+	char *endptr = NULL;
+	int bin = 0;
+
+	c = hist_str;
+
+	if (!dm_pool_begin_object(mem, sizeof(cur)))
+		return_0;
+
+	hist.nr_bins = nr_bins;
+
+	if (!dm_pool_grow_object(mem, &hist, sizeof(hist)))
+		goto_bad;
+
+	do {
+		memset(&cur, 0, sizeof(cur));
+		for (v = _valid_chars; *v; v++)
+			if (*c == *v)
+				break;
+		if (!*v)
+			goto badchar;
+
+		if (*c == ',')
+			goto badchar;
+		else {
+			val_start = c;
+			endptr = NULL;
+
+			this_val = strtoull(val_start, &endptr, 10);
+			if (!endptr) {
+				log_error("Could not parse histogram value.");
+				goto bad;
+			}
+			c = endptr; /* Advance to colon, or end. */
+
+			if (*c == ':')
+				c++;
+			else if (*c & (*c != '\n'))
+				/* Expected ':', '\n', or NULL. */
+				goto badchar;
+
+			if (*c == ':')
+				c++;
+
+			cur.upper = bounds->bins[bin].upper;
+			cur.count = this_val;
+			sum += this_val;
+
+			if (!dm_pool_grow_object(mem, &cur, sizeof(cur)))
+				goto_bad;
+
+			bin++;
+		}
+	} while (*c && (*c != '\n'));
+
+	log_debug("Added region histogram data with %d entries.", nr_bins);
+
+	*histogram = dm_pool_end_object(mem);
+	(*histogram)->sum = sum;
+
+	return 1;
+
+badchar:
+	log_error("Invalid character in histogram data: '%c' (0x%x)", *c, *c);
+bad:
+	dm_pool_abandon_object(mem);
+	return 0;
+}
+
+static int _stats_parse_region(struct dm_stats *dms, const char *resp,
 			       struct dm_stats_region *region,
 			       uint64_t timescale)
 {
+	struct dm_histogram *hist = NULL;
+	struct dm_pool *mem = dms->mem;
 	struct dm_stats_counters cur;
 	FILE *stats_rows = NULL;
 	uint64_t start, len;
@@ -453,7 +834,7 @@ static int _stats_parse_region(struct dm_pool *mem, const char *resp,
 	region->start = UINT64_MAX;
 
 	if (!dm_pool_begin_object(mem, 512))
-		goto_out;
+		goto_bad;
 
 	/*
 	 * dm_task_get_message_response() returns a 'const char *' but
@@ -461,7 +842,7 @@ static int _stats_parse_region(struct dm_pool *mem, const char *resp,
 	 */
 	stats_rows = fmemopen((char *)resp, strlen(resp), "r");
 	if (!stats_rows)
-		goto_out;
+		goto_bad;
 
 	/*
 	 * Output format for each step-sized area of a region:
@@ -509,7 +890,7 @@ static int _stats_parse_region(struct dm_pool *mem, const char *resp,
 			   &cur.total_read_nsecs, &cur.total_write_nsecs);
 		if (r != 15) {
 			log_error("Could not parse @stats_print row.");
-			goto out;
+			goto bad;
 		}
 
 		/* scale time values up if needed */
@@ -522,8 +903,33 @@ static int _stats_parse_region(struct dm_pool *mem, const char *resp,
 			cur.total_write_nsecs *= timescale;
 		}
 
-		if(!dm_pool_grow_object(mem, &cur, sizeof(cur)))
-			goto_out;
+		if (region->bounds) {
+			/* Find first histogram separator. */
+			char *hist_str = strchr(row, ':');
+			if (!hist_str) {
+				log_error("Could not parse histogram value.");
+				goto bad;
+			}
+			/* Find space preceding histogram. */
+			while (hist_str && *(hist_str - 1) != ' ')
+				hist_str--;
+
+			/* Use a separate pool for histogram objects since we
+			 * are growing the area table and each area's histogram
+			 * table simultaneously.
+			 */
+			if (!_stats_parse_histogram(dms->hist_mem, hist_str,
+						    &hist, region))
+				goto_bad;
+			hist->dms = dms;
+			hist->region = region;
+		}
+
+		cur.histogram = hist;
+
+		if (!dm_pool_grow_object(mem, &cur, sizeof(cur)))
+			goto_bad;
+
 		if (region->start == UINT64_MAX) {
 			region->start = start;
 			region->step = len; /* area size is always uniform. */
@@ -539,34 +945,15 @@ static int _stats_parse_region(struct dm_pool *mem, const char *resp,
 
 	return 1;
 
-out:
-
+bad:
 	if (stats_rows)
-		if(fclose(stats_rows))
+		if (fclose(stats_rows))
 			stack;
 	dm_pool_abandon_object(mem);
+
 	return 0;
 }
 
-static uint64_t _nr_areas(uint64_t len, uint64_t step)
-{
-	/* Default is one area. */
-	if (!len || !step)
-		return 1;
-	/*
-	 * drivers/md/dm-stats.c::message_stats_create()
-	 * A region may be sub-divided into areas with their own counters.
-	 * Any partial area at the end of the region is treated as an
-	 * additional complete area.
-	 */
-	return (len + step - 1) / step;
-}
-
-static uint64_t _nr_areas_region(struct dm_stats_region *region)
-{
-	return _nr_areas(region->len, region->step);
-}
-
 static void _stats_walk_next(const struct dm_stats *dms, int region,
 			     uint64_t *cur_r, uint64_t *cur_a)
 {
@@ -669,17 +1056,29 @@ uint64_t dm_stats_get_nr_areas(const struct dm_stats *dms)
 	return nr_areas;
 }
 
+int dm_stats_get_region_nr_histogram_bins(const struct dm_stats *dms,
+					  uint64_t region_id)
+{
+	region_id = (region_id == DM_STATS_REGION_CURRENT)
+		     ? dms->cur_region : region_id ;
+
+	if (!dms->regions[region_id].bounds)
+		return_0;
+
+	return dms->regions[region_id].bounds->nr_bins;
+}
+
 static int _stats_create_region(struct dm_stats *dms, uint64_t *region_id,
 				uint64_t start, uint64_t len, int64_t step,
-				int precise,
+				int precise, const char *hist_arg,
 				const char *program_id,	const char *aux_data)
 {
-	struct dm_task *dmt = NULL;
-	char msg[1024], range[64];
 	const char *err_fmt = "Could not prepare @stats_create %s.";
 	const char *precise_str = PRECISE_ARG;
 	const char *resp, *opt_args = NULL;
-	int nr_opt = 0, r = 0; /* number of optional args. */
+	char msg[1024], range[64], *endptr = NULL;
+	struct dm_task *dmt = NULL;
+	int r = 0, nr_opt = 0;
 
 	if (!_stats_bound(dms))
 		return_0;
@@ -703,9 +1102,16 @@ static int _stats_create_region(struct dm_stats *dms, uint64_t *region_id,
 	else
 		precise_str = "";
 
+	if (hist_arg)
+		nr_opt++;
+	else
+		hist_arg = "";
+
 	if (nr_opt) {
-		if ((dm_asprintf((char **)&opt_args, "%d %s", nr_opt,
-				 precise_str)) < 0) {
+		if ((dm_asprintf((char **)&opt_args, "%d %s %s%s", nr_opt,
+				 precise_str,
+				 (strlen(hist_arg)) ? HISTOGRAM_ARG : "",
+				 hist_arg)) < 0) {
 			log_error(err_fmt, PRECISE_ARG " option.");
 			return 0;
 		}
@@ -723,7 +1129,7 @@ static int _stats_create_region(struct dm_stats *dms, uint64_t *region_id,
 	}
 
 	if (!(dmt = _stats_send_message(dms, msg)))
-		goto out;
+		goto_out;
 
 	resp = dm_task_get_message_response(dmt);
 	if (!resp) {
@@ -732,7 +1138,6 @@ static int _stats_create_region(struct dm_stats *dms, uint64_t *region_id,
 	}
 
 	if (region_id) {
-		char *endptr = NULL;
 		*region_id = strtoull(resp, &endptr, 10);
 		if (resp == endptr)
 			goto_out;
@@ -741,42 +1146,38 @@ static int _stats_create_region(struct dm_stats *dms, uint64_t *region_id,
 	r = 1;
 
 out:
-	if(dmt)
+	if (dmt)
 		dm_task_destroy(dmt);
 	dm_free((void *) opt_args);
+
 	return r;
 }
 
 int dm_stats_create_region(struct dm_stats *dms, uint64_t *region_id,
 			   uint64_t start, uint64_t len, int64_t step,
-			   int precise,
-			   const char *program_id,
-			   const char *aux_data)
+			   int precise, struct dm_histogram *bounds,
+			   const char *program_id, const char *aux_data)
 {
-	/* Nanosecond counters need precise_timestamps. */
-	if (precise && !_stats_check_precise_timestamps(dms))
-		return 0;
+	char *hist_arg = NULL;
+	int r = 0;
 
-	return _stats_create_region(dms, region_id, start, len, step,
-				    precise,
-				    program_id, aux_data);
-}
+	/* Nanosecond counters and histograms both need precise_timestamps. */
+	if ((precise || bounds) && !_stats_check_precise_timestamps(dms))
+		return_0;
 
-/* Backward compatible dm_stats_create_region() */
-#if defined(__GNUC__)
-int dm_stats_create_region_v1_02_104(struct dm_stats *dms, uint64_t *region_id,
-				     uint64_t start, uint64_t len, int64_t step,
-				     const char *program_id, const char *aux_data);
-int dm_stats_create_region_v1_02_104(struct dm_stats *dms, uint64_t *region_id,
-				     uint64_t start, uint64_t len, int64_t step,
-				     const char *program_id, const char *aux_data)
-{
-	return _stats_create_region(dms, region_id, start, len, step,
-				    dms->precise,
-				    program_id, aux_data);
+	if (bounds) {
+		/* _build_histogram_arg enables precise if vals < 1ms. */
+		if (!(hist_arg = _build_histogram_arg(bounds, &precise)))
+			goto_out;
+	}
+
+	r = _stats_create_region(dms, region_id, start, len, step,
+				 precise, hist_arg, program_id, aux_data);
+	dm_free(hist_arg);
+
+out:
+	return r;
 }
-DM_EXPORT_SYMBOL(dm_stats_create_region, 1_02_104);
-#endif
 
 int dm_stats_delete_region(struct dm_stats *dms, uint64_t region_id)
 {
@@ -788,17 +1189,15 @@ int dm_stats_delete_region(struct dm_stats *dms, uint64_t region_id)
 
 	if (!dm_snprintf(msg, sizeof(msg), "@stats_delete " FMTu64, region_id)) {
 		log_error("Could not prepare @stats_delete message.");
-		goto out;
+		return 0;
 	}
 
 	dmt = _stats_send_message(dms, msg);
 	if (!dmt)
-		goto_out;
+		return_0;
 	dm_task_destroy(dmt);
-	return 1;
 
-out:
-	return 0;
+	return 1;
 }
 
 int dm_stats_clear_region(struct dm_stats *dms, uint64_t region_id)
@@ -811,50 +1210,47 @@ int dm_stats_clear_region(struct dm_stats *dms, uint64_t region_id)
 
 	if (!dm_snprintf(msg, sizeof(msg), "@stats_clear " FMTu64, region_id)) {
 		log_error("Could not prepare @stats_clear message.");
-		goto out;
+		return 0;
 	}
 
 	dmt = _stats_send_message(dms, msg);
-	if (!dmt)
-		goto_out;
-	dm_task_destroy(dmt);
-	return 1;
 
-out:
-	return 0;
+	if (!dmt)
+		return_0;
+
+	dm_task_destroy(dmt);
+
+	return 1;
 }
 
 static struct dm_task *_stats_print_region(struct dm_stats *dms,
 				    uint64_t region_id, unsigned start_line,
 				    unsigned num_lines, unsigned clear)
 {
-	struct dm_task *dmt = NULL;
 	/* @stats_print[_clear] <region_id> [<start_line> <num_lines>] */
 	const char *clear_str = "_clear", *lines_fmt = "%u %u";
 	const char *msg_fmt = "@stats_print%s " FMTu64 " %s";
 	const char *err_fmt = "Could not prepare @stats_print %s.";
+	struct dm_task *dmt = NULL;
 	char msg[1024], lines[64];
 
 	if (start_line || num_lines)
 		if (!dm_snprintf(lines, sizeof(lines),
 				 lines_fmt, start_line, num_lines)) {
 			log_error(err_fmt, "row specification");
-			goto out;
+			return NULL;
 		}
 
 	if (!dm_snprintf(msg, sizeof(msg), msg_fmt, (clear) ? clear_str : "",
 			 region_id, (start_line || num_lines) ? lines : "")) {
 		log_error(err_fmt, "message");
-		goto out;
+		return NULL;
 	}
 
 	if (!(dmt = _stats_send_message(dms, msg)))
-		goto out;
+		return_NULL;
 
 	return dmt;
-
-out:
-	return NULL;
 }
 
 char *dm_stats_print_region(struct dm_stats *dms, uint64_t region_id,
@@ -871,7 +1267,7 @@ char *dm_stats_print_region(struct dm_stats *dms, uint64_t region_id,
 				  start_line, num_lines, clear);
 
 	if (!dmt)
-		return 0;
+		return_0;
 
 	resp = dm_pool_strdup(dms->mem, dm_task_get_message_response(dmt));
 	dm_task_destroy(dmt);
@@ -890,7 +1286,7 @@ void dm_stats_buffer_destroy(struct dm_stats *dms, char *buffer)
 uint64_t dm_stats_get_nr_regions(const struct dm_stats *dms)
 {
 	if (!dms || !dms->regions)
-		return 0;
+		return_0;
 	return dms->nr_regions;
 }
 
@@ -900,10 +1296,10 @@ uint64_t dm_stats_get_nr_regions(const struct dm_stats *dms)
 int dm_stats_region_present(const struct dm_stats *dms, uint64_t region_id)
 {
 	if (!dms->regions)
-		return 0;
+		return_0;
 
 	if (region_id > dms->max_region)
-		return 0;
+		return_0;
 
 	return _stats_region_present(&dms->regions[region_id]);
 }
@@ -916,7 +1312,7 @@ static int _dm_stats_populate_region(struct dm_stats *dms, uint64_t region_id,
 	if (!_stats_bound(dms))
 		return_0;
 
-	if (!_stats_parse_region(dms->mem, resp, region, region->timescale)) {
+	if (!_stats_parse_region(dms, resp, region, region->timescale)) {
 		log_error("Could not parse @stats_print message response.");
 		return 0;
 	}
@@ -928,6 +1324,8 @@ int dm_stats_populate(struct dm_stats *dms, const char *program_id,
 		      uint64_t region_id)
 {
 	int all_regions = (region_id == DM_STATS_REGIONS_ALL);
+	struct dm_task *dmt = NULL; /* @stats_print task */
+	const char *resp;
 
 	if (!_stats_bound(dms))
 		return_0;
@@ -938,29 +1336,26 @@ int dm_stats_populate(struct dm_stats *dms, const char *program_id,
 
 	if (all_regions && !dm_stats_list(dms, program_id)) {
 		log_error("Could not parse @stats_list response.");
-		goto out;
+		goto bad;
 	}
 
 	/* successful list but no regions registered */
 	if (!dms->nr_regions)
-		return 0;
+		return_0;
 
 	dm_stats_walk_start(dms);
 	do {
-		struct dm_task *dmt = NULL; /* @stats_print task */
-		const char *resp;
-
 		region_id = (all_regions)
 			     ? dm_stats_get_current_region(dms) : region_id;
 
 		/* obtain all lines and clear counter values */
 		if (!(dmt = _stats_print_region(dms, region_id, 0, 0, 1)))
-			goto_out;
+			goto_bad;
 
 		resp = dm_task_get_message_response(dmt);
 		if (!_dm_stats_populate_region(dms, region_id, resp)) {
 			dm_task_destroy(dmt);
-			goto_out;
+			goto_bad;
 		}
 
 		dm_task_destroy(dmt);
@@ -970,7 +1365,7 @@ int dm_stats_populate(struct dm_stats *dms, const char *program_id,
 
 	return 1;
 
-out:
+bad:
 	_stats_regions_destroy(dms);
 	dms->regions = NULL;
 	return 0;
@@ -984,6 +1379,7 @@ void dm_stats_destroy(struct dm_stats *dms)
 	_stats_regions_destroy(dms);
 	_stats_clear_binding(dms);
 	dm_pool_destroy(dms->mem);
+	dm_pool_destroy(dms->hist_mem);
 	dm_free(dms->program_id);
 	dm_free(dms);
 }
@@ -1264,10 +1660,10 @@ int dm_stats_get_service_time(const struct dm_stats *dms, double *svctm,
 	double tput;
 
 	if (!dm_stats_get_throughput(dms, &tput, region_id, area_id))
-		return 0;
+		return_0;
 
 	if (!dm_stats_get_utilization(dms, &util, region_id, area_id))
-		return 0;
+		return_0;
 
 	/* avoid NAN with zero counter values */
 	if ( (uint64_t) tput == 0 || (uint64_t) util == 0) {
@@ -1501,3 +1897,466 @@ int dm_stats_get_current_region_precise_timestamps(const struct dm_stats *dms)
 {
 	return dm_stats_get_region_precise_timestamps(dms, dms->cur_region);
 }
+
+/*
+ * Histogram access methods.
+ */
+
+struct dm_histogram *dm_stats_get_histogram(const struct dm_stats *dms,
+					    uint64_t region_id,
+					    uint64_t area_id)
+{
+	region_id = (region_id == DM_STATS_REGION_CURRENT)
+		     ? dms->cur_region : region_id ;
+	area_id = (area_id == DM_STATS_AREA_CURRENT)
+		     ? dms->cur_area : area_id ;
+
+	if (!dms->regions[region_id].counters)
+		return dms->regions[region_id].bounds;
+
+	return dms->regions[region_id].counters[area_id].histogram;
+}
+
+int dm_histogram_get_nr_bins(const struct dm_histogram *dmh)
+{
+	return dmh->nr_bins;
+}
+
+uint64_t dm_histogram_get_bin_lower(const struct dm_histogram *dmh, int bin)
+{
+	return (!bin) ? 0 : dmh->bins[bin - 1].upper;
+}
+
+uint64_t dm_histogram_get_bin_upper(const struct dm_histogram *dmh, int bin)
+{
+	return dmh->bins[bin].upper;
+}
+
+uint64_t dm_histogram_get_bin_width(const struct dm_histogram *dmh, int bin)
+{
+	uint64_t upper, lower;
+	upper = dm_histogram_get_bin_upper(dmh, bin);
+	lower = dm_histogram_get_bin_lower(dmh, bin);
+	return (upper - lower);
+}
+
+uint64_t dm_histogram_get_bin_count(const struct dm_histogram *dmh, int bin)
+{
+	return dmh->bins[bin].count;
+}
+
+uint64_t dm_histogram_get_sum(const struct dm_histogram *dmh)
+{
+	return dmh->sum;
+}
+
+dm_percent_t dm_histogram_get_bin_percent(const struct dm_histogram *dmh,
+					  int bin)
+{
+	uint64_t value = dm_histogram_get_bin_count(dmh, bin);
+	uint64_t width = dm_histogram_get_bin_width(dmh, bin);
+	uint64_t total = dm_histogram_get_sum(dmh);
+
+	double val = (double) value;
+
+	if (!total || !value || !width)
+		return DM_PERCENT_0;
+
+	return dm_make_percent((uint64_t) val, total);
+}
+
+/*
+ * Histogram string helper functions: used to construct histogram and
+ * bin boundary strings from numeric data.
+ */
+
+/*
+ * Allocate an unbound histogram object with nr_bins bins. Only used
+ * for histograms used to hold bounds values as arguments for calls to
+ * dm_stats_create_region().
+ */
+static struct dm_histogram *_alloc_dm_histogram(int nr_bins)
+{
+	struct dm_histogram *dmh = NULL;
+	struct dm_histogram_bin *cur = NULL;
+	/* Allocate space for dm_histogram + nr_entries. */
+	size_t size = sizeof(*dmh) + (unsigned) nr_bins * sizeof(*cur);
+	return dm_zalloc(size);
+}
+
+/*
+ * Parse a histogram bounds string supplied by the user. The string
+ * consists of a list of numbers, "n1,n2,n3,..." with optional 'ns',
+ * 'us', 'ms', or 's' unit suffixes.
+ *
+ * The scale parameter indicates the timescale used for this region: one
+ * for nanoscale resolution and NSEC_PER_MSEC for miliseconds.
+ *
+ * On return bounds contains a pointer to an array of uint64_t
+ * histogram bounds values expressed in units of nanoseconds.
+ */
+struct dm_histogram *dm_histogram_bounds_from_string(const char *bounds_str)
+{
+	static const char *_valid_chars = "0123456789,muns";
+	uint64_t this_val = 0, mult = 1;
+	const char *c, *v, *val_start;
+	struct dm_histogram_bin *cur;
+	struct dm_histogram *dmh;
+	int nr_entries = 1;
+	char *endptr;
+
+	c = bounds_str;
+
+	/* Count number of bounds entries. */
+	while(*c)
+		if (*(c++) == ',')
+			nr_entries++;
+
+	c = bounds_str;
+
+	if (!(dmh = _alloc_dm_histogram(nr_entries)))
+		return_0;
+
+	dmh->nr_bins = nr_entries;
+
+	cur = dmh->bins;
+
+	do {
+		for (v = _valid_chars; *v; v++)
+			if (*c == *v)
+				break;
+
+		if (!*v) {
+			stack;
+			goto badchar;
+		}
+
+		if (*c == ',') {
+			log_error("Empty histogram bin not allowed: %s",
+				  bounds_str);
+			goto bad;
+		} else {
+			val_start = c;
+			endptr = NULL;
+
+			this_val = strtoull(val_start, &endptr, 10);
+			if (!endptr) {
+				log_error("Could not parse histogram bound.");
+				goto bad;
+			}
+			c = endptr; /* Advance to units, comma, or end. */
+
+			if (*c == 's') {
+				mult = NSEC_PER_SEC;
+				c++; /* Advance over 's'. */
+			} else if (*(c + 1) == 's') {
+				if (*c == 'm')
+					mult = NSEC_PER_MSEC;
+				else if (*c == 'u')
+					mult = NSEC_PER_USEC;
+				else if (*c == 'n')
+					mult = 1;
+				else {
+					stack;
+					goto badchar;
+				}
+				c += 2; /* Advance over 'ms', 'us', or 'ns'. */
+			} else if (*c == ',')
+				c++;
+			else if (*c) { /* Expected ',' or NULL. */
+				stack;
+				goto badchar;
+			}
+
+			if (*c == ',')
+				c++;
+			this_val *= mult;
+			(cur++)->upper = this_val;
+		}
+	} while (*c);
+
+	/* Bounds histograms have no owner. */
+	dmh->dms = NULL;
+	dmh->region = NULL;
+
+	return dmh;
+
+badchar:
+	log_error("Invalid character in histogram: %c", *c);
+bad:
+	dm_free(dmh);
+	return NULL;
+}
+
+struct dm_histogram *dm_histogram_bounds_from_uint64(const uint64_t *bounds)
+{
+	const uint64_t *entry = bounds;
+	struct dm_histogram_bin *cur;
+	struct dm_histogram *dmh;
+	int nr_entries = 1;
+
+	if (!bounds || !bounds[0]) {
+		log_error("Could not parse empty histogram bounds array");
+		return 0;
+	}
+
+	/* Count number of bounds entries. */
+	while(*entry)
+		if (*(++entry))
+			nr_entries++;
+
+	entry = bounds;
+
+	if (!(dmh = _alloc_dm_histogram(nr_entries)))
+		return_0;
+
+	dmh->nr_bins = nr_entries;
+
+	cur = dmh->bins;
+
+	while (*entry)
+		(cur++)->upper = *(entry++);
+
+	/* Bounds histograms have no owner. */
+	dmh->dms = NULL;
+	dmh->region = NULL;
+
+	return dmh;
+}
+
+void dm_histogram_bounds_destroy(struct dm_histogram *bounds)
+{
+	if (!bounds)
+		return;
+
+	/* Bounds histograms are not bound to any handle or region. */
+	if (bounds->dms || bounds->region) {
+		log_error("Freeing invalid histogram bounds pointer %p.",
+			  (void *) bounds);
+		stack;
+	}
+	/* dm_free() expects a (void *). */
+	dm_free((void *) bounds);
+}
+
+/*
+ * Scale a bounds value down from nanoseconds to the largest possible
+ * whole unit suffix.
+ */
+static void _scale_bound_value_to_suffix(uint64_t *bound, const char **suffix)
+{
+	*suffix = "ns";
+	if (!(*bound % NSEC_PER_SEC)) {
+		*bound /= NSEC_PER_SEC;
+		*suffix = "s";
+	} else if (!(*bound % NSEC_PER_MSEC)) {
+		*bound /= NSEC_PER_MSEC;
+		*suffix = "ms";
+	} else if (!(*bound % NSEC_PER_USEC)) {
+		*bound /= NSEC_PER_USEC;
+		*suffix = "us";
+	}
+}
+
+#define DM_HISTOGRAM_BOUNDS_MASK 0x30
+
+static int _make_bounds_string(char *buf, size_t size, uint64_t lower,
+			       uint64_t upper, int flags, int width)
+{
+	const char *l_suff = NULL;
+	const char *u_suff = NULL;
+	const char *sep = "";
+	char bound_buf[32];
+	int bounds = flags & DM_HISTOGRAM_BOUNDS_MASK;
+
+	if (!bounds)
+		return_0;
+
+	*buf = '\0';
+
+	if (flags & DM_HISTOGRAM_SUFFIX) {
+		_scale_bound_value_to_suffix(&lower, &l_suff);
+		_scale_bound_value_to_suffix(&upper, &u_suff);
+	} else
+		l_suff = u_suff = "";
+
+	if (flags & DM_HISTOGRAM_VALUES)
+		sep = ":";
+
+	if (bounds > DM_HISTOGRAM_BOUNDS_LOWER) {
+		/* Handle infinite uppermost bound. */
+		if (upper == UINT64_MAX) {
+			if (dm_snprintf(bound_buf, sizeof(bound_buf),
+					 ">" FMTu64 "%s", lower, l_suff) < 0)
+				goto_out;
+			/* Only display an 'upper' string for final bin. */
+			bounds = DM_HISTOGRAM_BOUNDS_UPPER;
+		} else {
+			if (dm_snprintf(bound_buf, sizeof(bound_buf),
+					 FMTu64 "%s", upper, u_suff) < 0)
+				goto_out;
+		}
+	} else if (bounds == DM_HISTOGRAM_BOUNDS_LOWER) {
+		if ((dm_snprintf(bound_buf, sizeof(bound_buf), FMTu64 "%s",
+				 lower, l_suff)) < 0)
+			goto_out;
+	}
+
+	switch (bounds) {
+	case DM_HISTOGRAM_BOUNDS_LOWER:
+	case DM_HISTOGRAM_BOUNDS_UPPER:
+		return dm_snprintf(buf, size, "%*s%s", width, bound_buf, sep);
+	case DM_HISTOGRAM_BOUNDS_RANGE:
+		return dm_snprintf(buf, size,  FMTu64 "%s-%s%s",
+				   lower, l_suff, bound_buf, sep);
+	}
+out:
+	return 0;
+}
+
+#define BOUND_WIDTH_NOSUFFIX 10 /* 999999999 nsecs */
+#define BOUND_WIDTH 6 /* bounds string up to 9999xs */
+#define COUNT_WIDTH 6 /* count string: up to 9999 */
+#define PERCENT_WIDTH 6 /* percent string : 0.00-100.00% */
+#define DM_HISTOGRAM_VALUES_MASK 0x06
+
+const char *dm_histogram_to_string(const struct dm_histogram *dmh, int bin,
+				   int width, int flags)
+{
+	int minwidth, bounds, values, start, last;
+	uint64_t lower, upper, val_u64; /* bounds of the current bin. */
+	/* Use the histogram pool for string building. */
+	struct dm_pool *mem = dmh->dms->hist_mem;
+	char buf[64], bounds_buf[64];
+	const char *sep = "";
+	int bounds_width;
+	ssize_t len = 0;
+	float val_flt;
+
+	bounds = flags & DM_HISTOGRAM_BOUNDS_MASK;
+	values = flags & DM_HISTOGRAM_VALUES;
+
+	if (bin < 0) {
+		start = 0;
+		last = dmh->nr_bins - 1;
+	} else
+		start = last = bin;
+
+	minwidth = width;
+
+	if (width < 0 || !values)
+		width = minwidth = 0; /* no padding */
+	else if (flags & DM_HISTOGRAM_PERCENT)
+		width = minwidth = (width) ? : PERCENT_WIDTH;
+	else if (flags & DM_HISTOGRAM_VALUES)
+		width = minwidth = (width) ? : COUNT_WIDTH;
+
+	if (values && !width)
+		sep = ":";
+
+	/* Set bounds string to the empty string. */
+	bounds_buf[0] = '\0';
+
+	if (!dm_pool_begin_object(mem, 64))
+		return_0;
+
+	for (bin = start; bin <= last; bin++) {
+		if (bounds) {
+			/* Default bounds width depends on time suffixes. */
+			bounds_width = (!(flags & DM_HISTOGRAM_SUFFIX))
+					? BOUND_WIDTH_NOSUFFIX
+					: BOUND_WIDTH ;
+
+			bounds_width = (!width) ? width : bounds_width;
+
+			lower = dm_histogram_get_bin_lower(dmh, bin);
+			upper = dm_histogram_get_bin_upper(dmh, bin);
+
+			len = sizeof(bounds_buf);
+			len = _make_bounds_string(bounds_buf, len,
+						  lower, upper, flags,
+						  bounds_width);
+			/*
+			 * Comma separates "bounds: value" pairs unless
+			 * --noheadings is used.
+			 */
+			sep = (width || !values) ? "," : ":";
+
+			/* Adjust width by real bounds length if set. */
+			width -= (width) ? (len - (bounds_width + 1)) : 0;
+
+			/* -ve width indicates specified width was overrun. */
+			width = (width > 0) ? width : 0;
+		}
+
+		if (bin == last)
+			sep = "";
+
+		if (flags & DM_HISTOGRAM_PERCENT) {
+			dm_percent_t pr;
+			pr = dm_histogram_get_bin_percent(dmh, bin);
+			val_flt = dm_percent_to_float(pr);
+			len = dm_snprintf(buf, sizeof(buf), "%s%*.2f%%%s",
+					  bounds_buf, width, val_flt, sep);
+		} else if (values) {
+			val_u64 = dmh->bins[bin].count;
+			len = dm_snprintf(buf, sizeof(buf), "%s%*"PRIu64"%s",
+					  bounds_buf, width, val_u64, sep);
+		} else if (bounds)
+			len = dm_snprintf(buf, sizeof(buf), "%s%s", bounds_buf,
+					  sep);
+
+		if (len < 0)
+			goto_bad;
+
+		width = minwidth; /* re-set histogram column width. */
+		if (!dm_pool_grow_object(mem, buf, (size_t) len))
+			goto_bad;
+	}
+
+	if (!dm_pool_grow_object(mem, "\0", 1))
+		goto_bad;
+
+	return (const char *) dm_pool_end_object(mem);
+
+bad:
+	dm_pool_abandon_object(mem);
+	return NULL;
+}
+
+/*
+ * Backward compatible dm_stats_create_region() implementations.
+ *
+ * Keep these at the end of the file to avoid adding clutter around the
+ * current dm_stats_create_region() version.
+ */
+
+#if defined(__GNUC__)
+int dm_stats_create_region_v1_02_106(struct dm_stats *dms, uint64_t *region_id,
+				     uint64_t start, uint64_t len, int64_t step,
+				     int precise, const char *program_id,
+				     const char *aux_data);
+int dm_stats_create_region_v1_02_106(struct dm_stats *dms, uint64_t *region_id,
+				     uint64_t start, uint64_t len, int64_t step,
+				     int precise, const char *program_id,
+				     const char *aux_data)
+{
+	/* 1.02.106 lacks histogram argument. */
+	return _stats_create_region(dms, region_id, start, len, step, precise,
+				    NULL, program_id, aux_data);
+}
+DM_EXPORT_SYMBOL(dm_stats_create_region, 1_02_106);
+
+int dm_stats_create_region_v1_02_104(struct dm_stats *dms, uint64_t *region_id,
+				     uint64_t start, uint64_t len, int64_t step,
+				     const char *program_id, const char *aux_data);
+int dm_stats_create_region_v1_02_104(struct dm_stats *dms, uint64_t *region_id,
+				     uint64_t start, uint64_t len, int64_t step,
+				     const char *program_id, const char *aux_data)
+{
+	/* 1.02.104 lacks histogram and precise arguments. */
+	return _stats_create_region(dms, region_id, start, len, step, 0, NULL,
+				    program_id, aux_data);
+}
+DM_EXPORT_SYMBOL(dm_stats_create_region, 1_02_104);
+#endif
