@@ -138,16 +138,16 @@ xx(lvchange,
    "Change the attributes of logical volume(s)",
    CACHE_VGMETADATA | PERMITTED_READ_ONLY,
    "lvchange\n"
-   "\t[-A|--autobackup y|n]\n"
-   "\t[-a|--activate [a|e|l]{y|n}]\n"
+   "\t[-A|--autobackup {y|n}]\n"
+   "\t[-a|--activate [a][e|s|l]{y|n}]\n"
    "\t[--activationmode {complete|degraded|partial}"
-   "\t[--addtag Tag]\n"
-   "\t[--alloc AllocationPolicy]\n"
-   "\t[-C|--contiguous y|n]\n"
-   "\t[--cachepolicy policyname] [--cachesettings parameter=value]\n"
-   "\t[--commandprofile ProfileName]\n"
+   "\t[--addtag <Tag>]\n"
+   "\t[--alloc <AllocationPolicy>]\n"
+   "\t[-C|--contiguous {y|n}]\n"
+   "\t[--cachepolicy <policyname>] [--cachesettings <parameter=value>]\n"
+   "\t[--commandprofile <ProfileName>]\n"
    "\t[-d|--debug]\n"
-   "\t[--deltag Tag]\n"
+   "\t[--deltag <Tag>]\n"
    "\t[--detachprofile]\n"
    "\t[--errorwhenfull {y|n}]\n"
    "\t[-f|--force]\n"
@@ -161,16 +161,16 @@ xx(lvchange,
    "\t[--monitor {y|n}]\n"
    "\t[--poll {y|n}]\n"
    "\t[--noudevsync]\n"
-   "\t[-M|--persistent y|n] [-j|--major major] [--minor minor]\n"
-   "\t[--metadataprofile ProfileName]\n"
+   "\t[-M|--persistent {y|n}] [-j|--major <major>] [--minor <minor>]\n"
+   "\t[--metadataprofile <ProfileName>]\n"
    "\t[-P|--partial]\n"
-   "\t[-p|--permission r|rw]\n"
-   "\t[--[raid]minrecoveryrate Rate]\n"
-   "\t[--[raid]maxrecoveryrate Rate]\n"
+   "\t[-p|--permission {r|rw}]\n"
+   "\t[--[raid]minrecoveryrate <Rate>]\n"
+   "\t[--[raid]maxrecoveryrate <Rate>]\n"
    "\t[--[raid]syncaction {check|repair}\n"
-   "\t[--[raid]writebehind IOCount]\n"
-   "\t[--[raid]writemostly PhysicalVolume[:{t|n|y}]]\n"
-   "\t[-r|--readahead ReadAheadSectors|auto|none]\n"
+   "\t[--[raid]writebehind <IOCount>]\n"
+   "\t[--[raid]writemostly <PhysicalVolume>[:{t|n|y}]]\n"
+   "\t[-r|--readahead <ReadAheadSectors>|auto|none]\n"
    "\t[--refresh]\n"
    "\t[--resync]\n"
    "\t[-S|--select Selection]\n"
@@ -180,7 +180,7 @@ xx(lvchange,
    "\t[--version]\n"
    "\t[-y|--yes]\n"
    "\t[-Z|--zero {y|n}]\n"
-   "\tLogicalVolume[Path] [LogicalVolume[Path]...]\n",
+   "\t<LogicalVolume[Path]> [<LogicalVolume[Path]>...]\n",
 
    activationmode_ARG, addtag_ARG, alloc_ARG, autobackup_ARG, activate_ARG,
    available_ARG, cachepolicy_ARG, cachesettings_ARG, contiguous_ARG, deltag_ARG,
